
//  Presidentit.cs  Copyright (c) 2005 Kari Laitinen

//  http://www.naturalprogramming.com

//  2005-01-23  Tiedosto luotu.
//  2005-02-11  Viimeisin muutos.

//  Kaannos: csc Presidentit.cs Paivamaara.cs

//  Vastaavan englanninkielisen ohjelman Presidents.cs:n
//  eiheena ovat USA:n presidentit joten sen President-luokassa
//  on myos varapresidenttia varten oleva datakentta.
//  Koska Suomessa ei ole varapresidenttia, tuo datakentta puuttuu
//  taman ohjelman Presidentti-luokasta. Muuten tama toimii
//  kuin vastaava englanninkielinen ohjelma. 

using System ;

class  Presidentti
{
   string      presidentin_nimi ;
   Paivamaara  presidentin_syntymaaika ;
   string      presidentin_syntymapaikka ;
   string      puolueen_nimi ;

   Paivamaara  virkaanastumispaiva ;
   Paivamaara  virastaluopumispaiva  ;

   public Presidentti( string  annettu_presidentin_nimi,
                       string  syntymaaika_stringina,
                       string  annettu_syntymapaikka,
                       string  annettu_puolueen_nimi,
                       string  virkaanastumispaiva_stringina,
                       string  virastaluopumispaiva_stringina )
   {
      presidentin_nimi  =   annettu_presidentin_nimi  ;
      presidentin_syntymaaika   =  new Paivamaara( syntymaaika_stringina ) ;
      presidentin_syntymapaikka =  annettu_syntymapaikka ;
      puolueen_nimi  =  annettu_puolueen_nimi ;
      virkaanastumispaiva   =  new Paivamaara( virkaanastumispaiva_stringina);
      virastaluopumispaiva  =  new Paivamaara( virastaluopumispaiva_stringina);
   }

   public string lue_presidentin_nimi()
   {
      return  presidentin_nimi ;
   }

   public bool oli_presidenttina( Paivamaara annettu_paiva )
   {
      return ( annettu_paiva.on_paivamaarien_sisalla( virkaanastumispaiva,
                                                      virastaluopumispaiva ));
   }

   public string lue_lyhyt_presidenttiinfo()
   {
      return ( "\n  "  +  presidentin_nimi.PadRight( 25 )
            +  " presidenttina "  +  virkaanastumispaiva
            +  " - " +  virastaluopumispaiva  ) ;
   }

   public string lue_kaikki_presidentin_tiedot()
   {
      int  vuodet_virassa, kuukaudet_virassa, paivat_virassa ;

      virkaanastumispaiva.laske_etaisyys( virastaluopumispaiva,
                                          out  vuodet_virassa,
                                          out  kuukaudet_virassa,
                                          out  paivat_virassa ) ;
      return (  "\n  "  +  presidentin_nimi
           +  "\n    Syntymaaika:  "   +  presidentin_syntymaaika
           +  "   Syntymapaikka: "  +  presidentin_syntymapaikka  
           +  "\n    Aloitti presidenttina : "  +  virkaanastumispaiva 
           +  "\n    Lopetti presidenttina : "  +  virastaluopumispaiva
           +  "\n    Toimi presidenttina " +  vuodet_virassa
           +  " vuotta, "  +  kuukaudet_virassa  + " kuukautta ja "
           +  paivat_virassa  +  " paivaa."
           +  "\n    Presidentin puolue: "  +  puolueen_nimi ) ;
   }
}


class  Presidenttitietosovellus
{
   Presidentti[]  presidenttitaulukko  =  new  Presidentti[ 80 ] ;

   int  presidentteja_taulukossa ;
   int  viimeisen_tulostuksen_indeksi ;

   const int  ETSINTA_EI_VALMIS    =  1 ;
   const int  ETSINTA_ON_VALMIS    =  2 ;
   const int  ETSINTA_TULOKSEKAS   =  3 ;
   const int  ETSINTA_TULOKSETON   =  4 ;

   /*  Seuraavan taulukon tiedot on keratty erinaisista Internet-
       yms. lahteista. Presidenttien syntymaajat ja paikat pitaisi
       olla oikein, mutta virkaanastumispaivat eivat ole taysin
       varmasti oikein. Vuodet ovat kuitenkin oikein.

       Yleensa Suomen tasavallan presidentit ovat astuneet virkaan
       vuoden alkupuolella kun presidentinvaali on ollut
       tammikuun puolivalissa. Viimeisimmat presidentit ovat astuneet
       virkaan 01.03. joten tuota paivaa on taulukossa kaytetty
       vanhempienkin presidenttien kohdalla.

       Muutamia tapahtumia:

       15.-16.1.1937 on ollut presidentin valitsijamiesvaalit.
       Nuita valitsijamiehia kaytettiin useaan kertaan koska
       sodan takia ei voitu vaaleja pitaa.

       19.12.1940 eduskunta valitsi Risto Rytin presidentiksi
       Kallion kautta jatkamaan. Kallio oli pyytanyt eroa jo
       27.11.1940. Kallio kuoli 19.12.1940.

       4.3.1946 eduskunta valitsi Paasikiven presidentiksi.
   */

   public Presidenttitietosovellus()
   {
      presidenttitaulukko[ 0 ]  =  new
       Presidentti( "Kaarlo Juho Stahlberg",   "28.01.1865",  "Suomussalmi",
       "Keskusta-oikeisto", "26.07.1919", "01.03.1925" ) ;
      presidenttitaulukko[ 1 ]  =  new
       Presidentti( "Lars Kristian Relander",  "31.05.1883", "Kurkijoki",
       "Maalaisliitto", "01.03.1925", "01.03.1931" ) ;
      presidenttitaulukko[ 2 ]  =  new
       Presidentti( "Pehr Evind Svinhufvud", "15.12.1861", "Saaksmaen Rapola",
       "Oikeisto-keskusta", "01.03.1931", "01.03.1937" ) ; 
      presidenttitaulukko[ 3 ]  =  new
       Presidentti( "Kyosti Kallio",  "10.04.1873",  "Ylivieska",
       "Maalaisliitto", "01.03.1937", "19.12.1940" ) ;
      presidenttitaulukko[ 4 ]  =  new
       Presidentti( "Risto Heikki Ryti", "03.02.1889", "Huittinen",
       "Edistyspuolue", "19.12.1940", "01.08.1944" );
      presidenttitaulukko[ 5 ]  =  new
       Presidentti( "Carl Gustaf Emil Mannerheim", "04.06.1867", "Asikainen",
       "Puolueista ulkona", "04.08.1944", "04.03.1946" ) ;
      presidenttitaulukko[ 6 ]  =  new
       Presidentti( "Juho Kusti Paasikivi", "27.11.1870", "Kosken pitaja",
       "Kokoomus", "09.03.1946", "01.03.1956" );
      presidenttitaulukko[ 7 ]  =  new
       Presidentti( "Urho Kaleva Kekkonen", "03.09.1900", "Pielavesi",
       "Maalaisliitto", "01.03.1956", "01.03.1982" );
      presidenttitaulukko[ 8 ]  =  new
       Presidentti( "Mauno Henrik Koivisto", "25.11.1923", "Turku",
       "Sosiaalidemokraatit", "01.03.1982", "01.03.1994" );
      presidenttitaulukko[ 9 ]  =  new
       Presidentti( "Martti Oiva Kalevi Ahtisaari", "23.06.1937", "Viipuri",
       "Sosiaalidemokraatit", "01.03.1994", "01.03.2000" );
      presidenttitaulukko[ 10 ]  =  new
       Presidentti( "Tarja Kaarina Halonen", "24.12.1943", "Helsinki",
       "Sosiaalidemokraatit", "01.03.2000", "01.03.2006" );

      //  Seuraavan muuttujan arvo pitaa muuttaa, kun
      //  presidenttitaulukkoon lisataan uusia presidentteja.

      presidentteja_taulukossa  =  11 ; 

      viimeisen_tulostuksen_indeksi  =  0 ;
   }

   public void etsi_presidenttia_nimella()
   {
      Console.Write( "\n  Anna presidentin nimi: ") ;

      string  annettu_presidentin_nimi  =  Console.ReadLine() ;

      int  presidentin_indeksi  =  0 ;
      int  taulukkohaun_tila    =  ETSINTA_EI_VALMIS ;

      while ( taulukkohaun_tila == ETSINTA_EI_VALMIS )
      {
         if ( presidenttitaulukko[ presidentin_indeksi ].lue_presidentin_nimi()
              .IndexOf( annettu_presidentin_nimi )  !=  -1 )
         {
            taulukkohaun_tila  =  ETSINTA_TULOKSEKAS ;
         }
         else if ( presidentin_indeksi  >=  presidentteja_taulukossa - 1 )
         {
            taulukkohaun_tila  =  ETSINTA_TULOKSETON ;
         }
         else
         {
            presidentin_indeksi  ++  ;
         }
      }

      if ( taulukkohaun_tila  ==  ETSINTA_TULOKSEKAS )
      {
         Console.Write( "\n\n  SUOMEN TASAVALLAN "
              +  ( presidentin_indeksi + 1 )  +  ". PRESIDENTTI:  \n" 

              +  presidenttitaulukko[ presidentin_indeksi ].
                                 lue_kaikki_presidentin_tiedot() ) ;

         viimeisen_tulostuksen_indeksi  =  presidentin_indeksi ;
      }
      else
      {
         Console.Write( "\n\n  Valitettavasti nimella \""
              +  annettu_presidentin_nimi  +  "\" ei loydy tietoja.\n" ) ;
      }
   }

   public void etsi_presidenttia_paivamaaralle()
   {
      Console.Write( "\n Anna paivamaara muodossa PP.KK.VVVV "
                  +  "\n (Siis paiva ja kuukausi kaksinumeroisesti ja"
                  +  "\n vuosi nelinumeroisesti pistein eroteltuna): " ) ;

      string  paivamaara_stringina  =  Console.ReadLine() ;

      Paivamaara  annettu_paivamaara 
                      =  new  Paivamaara( paivamaara_stringina ) ;

      int  presidentin_indeksi  =  0 ;
      int  taulukkohaun_tila    =  ETSINTA_EI_VALMIS ;

      while ( taulukkohaun_tila == ETSINTA_EI_VALMIS )
      {
         if ( presidenttitaulukko[ presidentin_indeksi ].
                               oli_presidenttina( annettu_paivamaara ) )
         {
            taulukkohaun_tila  =  ETSINTA_TULOKSEKAS ;
         }
         else if ( presidentin_indeksi  >=  presidentteja_taulukossa - 1)
         {
            taulukkohaun_tila  =  ETSINTA_TULOKSETON ;
         }
         else
         {
            presidentin_indeksi  ++  ;
         }
      }

      if ( taulukkohaun_tila  ==  ETSINTA_TULOKSEKAS )
      {
         Console.Write( "\n\n  "   +  annettu_paivamaara
              + " SUOMEN TASAVALLAN PRESIDENTTI OLI: \n"

              + presidenttitaulukko[ presidentin_indeksi ].
                                lue_kaikki_presidentin_tiedot() ) ;

         viimeisen_tulostuksen_indeksi  =  presidentin_indeksi ;
      }
      else
      {
         Console.Write( "\n\n  Presidenttia ei ollut virassa "
                     +  annettu_paivamaara  + ".\n" ) ;
      }
   }

   public void tulosta_seuraavan_presidentin_tiedot()
   {
      viimeisen_tulostuksen_indeksi   ++  ;

      if ( viimeisen_tulostuksen_indeksi  <  presidentteja_taulukossa )
      {
         Console.Write( "\n\n  SUOMEN TASAVALLAN "
             +  ( viimeisen_tulostuksen_indeksi + 1 )  +  ". PRESIDENTTI:\n" 

             + presidenttitaulukko[ viimeisen_tulostuksen_indeksi ].
                                       lue_kaikki_presidentin_tiedot() ) ;
      }
      else
      {
         Console.Write( "\n Ei enempaa tietoja saatavilla.\n" ) ;
      }
   }

   public void tulosta_lista_kaikista_presidenteista()
   {
      int  presidentin_indeksi  =  0 ;

      while ( presidentin_indeksi  <  presidentteja_taulukossa )
      {
         Console.Write( presidenttitaulukko[ presidentin_indeksi ].
                                           lue_lyhyt_presidenttiinfo() ) ;
         presidentin_indeksi  ++ ;

         if ( ( presidentin_indeksi  %  15 ) ==  0 )
         {
            Console.Write( "\nPaina Enter jatkaaksesi ....." ) ;
            string  mika_tahansa_stringi  =  Console.ReadLine() ;
         }
      }
   }


   public void suorita()
   {
      string  valittu_toiminto  =  "????"  ;

      Console.Write("\n Tama ohjelma antaa tietoa Suomen tasavallan"
                 +  "\n presidenteista. Voit valita toiminnon"
                 +  "\n kirjaimen avulla seuraavan menun mukaisesti. ") ;

      while ( valittu_toiminto[ 0 ]  !=  'x' )
      {
         Console.Write("\n\n   n   Etsi presidenttia nimella."
                      +  "\n   p   Etsi presidenttia paivamaaralle."
                      +  "\n   s   Tulosta seuraavan presidentin tiedot."
                      +  "\n   k   Tulosta lista kaikista presidenteista."
                      +  "\n   x   Poistu ohjelmasta.\n\n   " ) ;

         valittu_toiminto  =  Console.ReadLine() ;

         if ( valittu_toiminto[ 0 ]  ==  'n' )
         {
            etsi_presidenttia_nimella() ;
         }
         else if ( valittu_toiminto[ 0 ]  ==  'p' )
         {
            etsi_presidenttia_paivamaaralle() ;
         }
         else if ( valittu_toiminto[ 0 ]  ==  's' )
         {
            tulosta_seuraavan_presidentin_tiedot() ;
         }
         else if ( valittu_toiminto[ 0 ]  ==  'k' )
         {
            tulosta_lista_kaikista_presidenteista() ;
         }
      }
   }
}


class  PresidenttitietosovelluksenSuorittaja
{
   static void Main()
   {
      Presidenttitietosovellus  tama_presidenttitietosovellus
                                  = new Presidenttitietosovellus() ;

      tama_presidenttitietosovellus.suorita() ;
   }
}



